<?php

namespace Ymina\Checkout\Helper;

use Laminas\ReCaptcha\Exception;
use Magento\Framework\App\Helper\AbstractHelper;
use Magento\Framework\App\Helper\Context;

class RutUtility extends AbstractHelper
{
    const DOTS = 0;
    const DASH = 1;
    const DOTS_DASH = 2;

    private $regexR = '/[.-]/';

    public function __construct(Context $context)
    {
        parent::__construct($context);
    }

    public  function rutLikePattern()
    {
        return '/^(\d{0,2})\.?(\d{3})\.?(\d{3})-?(\d|k)$/i';
    }

    public function suspiciousRutPattern()
    {
        return '/^(\d)\1?\.?(\1{3})\.?(\1{3})-?(\d|k)?/i';
    }

    public function isRutLike($rut)
    {
        return preg_match($this->rutLikePattern(), $rut);
    }

    public function isSuspiciousRut($rut)
    {
        return preg_match($this->suspiciousRutPattern(), $rut);
    }

    public function cleanRut($rut)
    {
        return $this->isRutLike($rut) ? preg_replace('/[^0-9k]/i', '', $rut) : '';
    }

    public  function getRutDigits($rut)
    {
        return substr(self::cleanRut($rut), 0, -1);
    }

    public function getRutVerifier($rut)
    {
        return substr($this->cleanRut($rut), -1);
    }

    public function deconstructRut($rut)
    {
        return [
            'digits' => $this->getRutDigits($rut),
            'verifier' => $this->getRutVerifier($rut),
        ];
    }

    public function resetRut($rut)
    {
        return preg_replace($this->regexR, '', $rut);
    }

    public function formatRut($rut, $format = self::DASH)
    {
        if ($format === null) {
            $format = self::DASH;
        }
        if ($rut === null || $rut === '') {
            return '';
        }
        if (!is_string($rut)) {
            throw new Exception('RUT needs to be a string or undefined');
        }
        if (!self::isRutLike($rut)) {
            return $rut;
        }
        switch ($format) {
            case self::DOTS:
                return preg_replace_callback($this->rutLikePattern(), function ($matches) {
                    return ($matches[1] ? "{$matches[1]}." : '') . "{$matches[2]}.{$matches[3]}{$matches[4]}";
                }, $rut);
            case self::DASH:
                return preg_replace($this->rutLikePattern(), '$1$2$3-$4', $rut);
            case self::DOTS_DASH:
                return preg_replace_callback($this->rutLikePattern(), function ($matches) {
                    return ($matches[1] ? "{$matches[1]}." : '') . "{$matches[2]}.{$matches[3]}-{$matches[4]}";
                }, $rut);
            default:
                return preg_replace($this->rutLikePattern(), '$1$2$3$4', $rut);
        }
    }

    public function calculateRutVerifier($digits)
    {
        $sum = 0;
        $mul = 2;
        $length = strlen($digits);
        for ($i = $length - 1; $i >= 0; $i--) {
            $sum += intval($digits[$i]) * $mul;
            if ($mul % 7 === 0) {
                $mul = 2;
            } else {
                $mul++;
            }
        }
        $res = $sum % 11;
        if ($res === 0) {
            return '0';
        } elseif ($res === 1) {
            return 'k';
        }
        return strval(11 - $res);
    }

    public function validateRut($rut, $noSuspicious = true)
    {
        if (!$this->isRutLike($rut)) {
            return false;
        }
        if ($noSuspicious && $this->isSuspiciousRut($rut)) {
            return false;
        }
        return strtolower($this->getRutVerifier($rut)) === $this->calculateRutVerifier($this->getRutDigits($rut));
    }

    /**
     * @param $value
     * @return bool
     */
    public function validateRutV1($value)
    {
        $value = $this->resetRut($value);
        $value = self::formatRut($value, self::DOTS_DASH);

        $value = str_replace('.', '', $value);
        $tmp = explode('-', $value);

        if (count($tmp) !== 2) {
            return false;
        }

        $digv = $tmp[1];
        $rut = $tmp[0];
        if (strlen($rut) < 7 || strlen($rut) > 9) {
            return false;
        }
        if ($digv === 'K') {
            $digv = 'k';
        }
        return $this->dv($rut) == $digv;
    }

    private function dv($T)
    {
        $M = 0;
        $S = 1;
        for (; $T; $T = floor($T / 10)) {
            $S = ($S + $T % 10 * (9 - $M++ % 6)) % 11;
        }

        return $S ? strval($S - 1) : 'k';
    }

    public function validateRutList($ruts, $noSuspicious = true)
    {
        $res = [];
        foreach ($ruts as $rut) {
            $res[$rut] = $this->validateRut($rut, $noSuspicious);
        }
        return $res;
    }

    public  function generateRut()
    {
        $digits = strval(10000003 + rand(0, 90000000));
        $verifier = $this->calculateRutVerifier($digits);
        return $this->formatRut($digits . $verifier);
    }
}