define([], function () {
    const RutFormat = {
        DOTS: 0,
        DASH: 1,
        DOTS_DASH: 2
    };

    const regexR = /[.-]/g;

    return {

        /**
         *
         * @returns {RegExp}
         */
        rutLikePattern: function () {
            return /^(\d{0,2})\.?(\d{3})\.?(\d{3})-?(\d|k)$/gi;
        },


        /**
         *
         * @returns {RegExp}
         */
        suspiciousRutPattern: function () {
            return /^(\d)\1?\.?(\1{3})\.?(\1{3})-?(\d|k)?$/gi;
        },


        /**
         *
         * @param rut
         * @returns {boolean}
         */
        isRutLike: function (rut) {
            return this.rutLikePattern().test(rut);
        },


        /**
         *
         * @param rut
         * @returns {boolean}
         */
        isSuspiciousRut: function (rut) {
            return this.suspiciousRutPattern().test(rut);
        },


        /**
         *
         * @param rut
         * @returns {*|string}
         */
        cleanRut: function (rut) {
            return (this.isRutLike(rut) ? rut.replace(/[^0-9k]/gi, '') : '');
        },


        /**
         *
         * @param rut
         * @returns {string}
         */
        getRutDigits: function (rut) {
            return this.cleanRut(rut).slice(0, -1);
        },


        /**
         *
         * @param rut
         * @returns {string}
         */
        getRutVerifier: function (rut) {
            return this.cleanRut(rut).slice(-1);
        },

        /**
         *
         * @param rut
         * @returns {{verifier: string, digits: string}}
         */
        deconstructRut: function (rut) {
            return ({
                digits: this.getRutDigits(rut),
                verifier: this.getRutVerifier(rut),
            });
        },

        /**
         *
         * @param rut
         * @returns {*}
         */
        resetRut: function (rut) {
            return rut.replace(regexR, '');
        },

        /**
         *
         * @param rut
         * @param format
         * @returns {string}
         */
        formatRut: function (rut, format) {
            if (format === void 0) {
                format = RutFormat.DASH;
            }
            if (rut === null || rut === undefined)
                return '';
            if (typeof rut !== 'string')
                throw new TypeError('RUT needs to be a string or undefined');
            if (!this.isRutLike(rut))
                return rut;
            switch (format) {
                case RutFormat.DOTS:
                    return rut.replace(this.rutLikePattern(), function () {
                        var m = [];
                        for (var _i = 0; _i < arguments.length; _i++) {
                            m[_i] = arguments[_i];
                        }
                        return "".concat(m[1] ? "".concat(m[1], ".") : '').concat(m[2], ".").concat(m[3]).concat(m[4]);
                    });
                case RutFormat.DASH:
                    return rut.replace(this.rutLikePattern(), '$1$2$3-$4');
                case RutFormat.DOTS_DASH:
                    return rut.replace(this.rutLikePattern(), function () {
                        var m = [];
                        for (var _i = 0; _i < arguments.length; _i++) {
                            m[_i] = arguments[_i];
                        }
                        return "".concat(m[1] ? "".concat(m[1], ".") : '').concat(m[2], ".").concat(m[3], "-").concat(m[4]);
                    });
                default:
                    return rut.replace(this.rutLikePattern(), '$1$2$3$4');
            }
        },


        /**
         *
         * @param digits
         * @returns {string}
         */
        calculateRutVerifier: function (digits) {
            var sum = 0;
            var mul = 2;
            var i = digits.length;
            while (i--) {
                sum = sum + parseInt(digits.charAt(i)) * mul;
                if (mul % 7 === 0) {
                    mul = 2;
                } else {
                    mul++;
                }
            }
            var res = sum % 11;
            if (res === 0) {
                return '0';
            } else if (res === 1) {
                return 'k';
            }
            return "".concat(11 - res);
        },


        /**
         *
         * @param rut
         * @param noSuspicious
         * @returns {boolean}
         */
        validateRut: function (rut, noSuspicious) {
            if (noSuspicious === void 0) {
                noSuspicious = true;
            }
            if (!this.isRutLike(rut))
                return false;
            if (noSuspicious && this.isSuspiciousRut(rut))
                return false;
            return this.getRutVerifier(rut).toLowerCase() === this.calculateRutVerifier(this.getRutDigits(rut));
        },


        /**
         *
         * @param value
         * @returns {boolean}
         */
        validateRutV1: function (value) {
            value = this.resetRut(value);
            value = this.formatRut(value, RutFormat.DOTS_DASH);
            // console.log(value)

            value = value.replace(/\./g, '');
            var tmp = value.split('-');
            var digv = tmp[1];
            var rut = tmp[0];
            if (rut.length < 7 || rut.length > 9) {
                return false;
            }
            if (digv == 'K') {
                digv = 'k';
            }
            return (this.dv(rut) == digv);
        },

        dv: function (T) {
            var M = 0, S = 1;
            for (; T; T = Math.floor(T / 10))
                S = (S + T % 10 * (9 - M++ % 6)) % 11;

            return S ? S - 1 : 'k';
        },


        /**
         *
         * @param ruts
         * @param noSuspicious
         * @returns {Map<any, any>}
         */
        validateRutList: function (ruts, noSuspicious) {
            if (noSuspicious === void 0) {
                noSuspicious = true;
            }
            var res = new Map();
            for (var _i = 0, ruts_1 = ruts; _i < ruts_1.length; _i++) {
                var rut = ruts_1[_i];
                res.set(rut, this.validateRut(rut, noSuspicious));
            }
            return res;
        },


        /**
         *
         * @returns {string}
         */
        generateRut: function () {
            var digits = Math.floor(10000003 + Math.random() * 90000000).toString();
            var verifier = this.calculateRutVerifier(digits);
            return this.formatRut(digits + verifier);
        }
    }

})