<?php
/**
 * Copyright ©  All rights reserved.
 * See COPYING.txt for license details.
 */
declare(strict_types=1);

namespace Ymina\Customer\Rewrite\Magento\Customer\Model\Address\Validator;


use Magento\Customer\Api\AddressMetadataInterface;
use Magento\Customer\Model\Address\AbstractAddress;
use Magento\Customer\Model\Address\ValidatorInterface;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Validator\NotEmpty;
use Magento\Framework\Validator\ValidateException;
use Magento\Framework\Validator\ValidatorChain;

/**
 * Address general fields validator.
 */
class General implements ValidatorInterface
{
    /**
     * @var \Magento\Eav\Model\Config
     */
    private $eavConfig;

    /**
     * @var \Magento\Directory\Helper\Data
     */
    private $directoryData;


    /**
     * @var \Ymina\Checkout\Helper\RutUtility
     */
    private $rutUtility;

    /**
     * @param \Magento\Eav\Model\Config $eavConfig
     * @param \Magento\Directory\Helper\Data $directoryData
     * @param \Ymina\Checkout\Helper\RutUtility $rutUtility
     */
    public function __construct(
        \Magento\Eav\Model\Config $eavConfig,
        \Magento\Directory\Helper\Data $directoryData,
        \Ymina\Checkout\Helper\RutUtility $rutUtility
    ) {
        $this->eavConfig = $eavConfig;
        $this->directoryData = $directoryData;
        $this->rutUtility = $rutUtility;
    }

    /**
     * @inheritdoc
     */
    public function validate(AbstractAddress $address)
    {
        $errors = array_merge(
            $this->checkRequiredFields($address),
            $this->checkOptionalFields($address)
        );

        return $errors;
    }

    /**
     * Check fields that are generally required.
     *
     * @param AbstractAddress $address
     * @return array
     * @throws ValidateException
     */
    private function checkRequiredFields(AbstractAddress $address)
    {
        $errors = [];
        if (!ValidatorChain::is($address->getFirstname(), NotEmpty::class)) {
            $errors[] = __('"%fieldName" is required. Enter and try again.', ['fieldName' => 'nombre']);
        }

        if (!ValidatorChain::is($address->getLastname(), NotEmpty::class)) {
            $errors[] = __('"%fieldName" is required. Enter and try again.', ['fieldName' => 'apellido']);
        }

        if (!ValidatorChain::is($address->getStreetLine(1), NotEmpty::class)) {
            $errors[] = __('"%fieldName" is required. Enter and try again.', ['fieldName' => 'dirección']);
        }

        if (!ValidatorChain::is($address->getStreetLine(2), NotEmpty::class)) {
            $errors[] = __('"%fieldName" is required. Enter and try again.', ['fieldName' => 'número de domicilio']);
        }

        if (!ValidatorChain::is($address->getCity(), NotEmpty::class)) {
            $errors[] = __('"%fieldName" is required. Enter and try again.', ['fieldName' => 'comuna']);
        }

        if (!ValidatorChain::is($address->getVatId(), NotEmpty::class)) {
            $errors[] = __('"%fieldName" is required. Enter and try again.', ['fieldName' => 'número de documento']);
        }

        if ($address->getFax() == "RUT") {
            if (!$this->rutUtility->validateRutV1($address->getVatId())) {
                $errors[] = __('"%fieldName" es inválido. Ingresa y vuelve a intentarlo.', ['fieldName' => 'número de documento']);
            }
        }

        return $errors;
    }

    /**
     * Check fields that are conditionally required.
     *
     * @param AbstractAddress $address
     * @return array
     * @throws LocalizedException|ValidateException
     */
    private function checkOptionalFields(AbstractAddress $address)
    {
        $this->reloadAddressAttributes($address);
        $errors = [];
        if ($this->isTelephoneRequired()
            && !ValidatorChain::is($address->getTelephone(), NotEmpty::class)
        ) {
            $errors[] = __('"%fieldName" is required. Enter and try again.', ['fieldName' => 'número de teléfono']);
        }

        if ($this->isFaxRequired()
            && !ValidatorChain::is($address->getFax(), NotEmpty::class)
        ) {
            $errors[] = __('"%fieldName" is required. Enter and try again.', ['fieldName' => 'tipo de documento']);
        }

        if ($this->isCompanyRequired()
            && !ValidatorChain::is($address->getCompany(), NotEmpty::class)
        ) {
            $errors[] = __('"%fieldName" is required. Enter and try again.', ['fieldName' => 'company']);
        }

        $havingOptionalZip = $this->directoryData->getCountriesWithOptionalZip();
        if (!in_array($address->getCountryId(), $havingOptionalZip)
            && !ValidatorChain::is($address->getPostcode(), NotEmpty::class)
        ) {
            $errors[] = __('"%fieldName" is required. Enter and try again.', ['fieldName' => 'postcode']);
        }

        return $errors;
    }

    /**
     * Check if company field required in configuration.
     *
     * @return bool
     * @throws LocalizedException
     */
    private function isCompanyRequired()
    {
        return $this->eavConfig->getAttribute('customer_address', 'company')->getIsRequired();
    }

    /**
     * Check if telephone field required in configuration.
     *
     * @return bool
     * @throws LocalizedException
     */
    private function isTelephoneRequired()
    {
        return $this->eavConfig->getAttribute('customer_address', 'telephone')->getIsRequired();
    }

    /**
     * Check if fax field required in configuration.
     *
     * @return bool
     * @throws LocalizedException
     */
    private function isFaxRequired()
    {
        return $this->eavConfig->getAttribute('customer_address', 'fax')->getIsRequired();
    }

    /**
     * Reload address attributes for the certain store
     *
     * @param AbstractAddress $address
     * @return void
     */
    private function reloadAddressAttributes(AbstractAddress $address): void
    {
        $attributeSetId = $address->getAttributeSetId() ?: AddressMetadataInterface::ATTRIBUTE_SET_ID_ADDRESS;
        $address->setData('attribute_set_id', $attributeSetId);
        $this->eavConfig->getEntityAttributes(AddressMetadataInterface::ENTITY_TYPE_ADDRESS, $address);
    }

    public function validateRut($rut){
        // Formato de expresión regular para validar un RUT chileno
        $patron = '/^\d{1,2}\.\d{3}\.\d{3}[-][0-9kK]$/i';

        // Eliminar puntos y guión del RUT para realizar la validación
        $rut = str_replace(array('.', '-'), '', $rut);

        if (preg_match($patron, $rut)) {
            // Validar el dígito verificador
            $rut = strtoupper($rut);
            $rut = str_replace("K", "k", $rut);
            list($rut, $dv) = explode('-', $rut);
            $dvr = '0';
            for ($i = 0; $i < strlen($rut); $i++) {
                $dvr = ($dvr + (int)($rut[$i]) * (9 - $i)) % 11;
            }
            if ($dv == $dvr || $dv == (11 - $dvr)) {
                return true; // RUT válido
            }
        }

        return false; // RUT inválido
    }

}